Package: sftime
Title: Classes and Methods for Simple Feature Objects that Have a Time
        Column
Description: Classes and methods for spatial objects that have a registered time 
    column, in particular for irregular spatiotemporal data. The time column can 
    be of any type, but needs to be ordinal. Regularly laid out spatiotemporal 
    data (vector or raster data cubes) are handled by package 'stars'. 
Version: 0.3.2
Depends: R (>= 4.1.0), sf (>= 1.0.9)
Imports: methods
Suggests: knitr, spacetime, rmarkdown, dplyr (>= 0.8-3), trajectories
        (>= 0.2.2), stars, ncmeta, tidyr, ggplot2, sp, rlang, vctrs,
        spatstat.geom, spatstat.linnet, sftrack, cubble (>= 0.3.0)
Authors@R: 
    c(person(given = "Henning",
             family = "Teickner",
             role = c("aut", "cre", "cph"),
             email = "henning.teickner@uni-muenster.de",
             comment = c(ORCID = "0000-0002-3993-1182")),
      person(given = "Edzer",
             family = "Pebesma",
             role = c("aut", "cph"),
             email = "edzer.pebesma@uni-muenster.de",
             comment = c(ORCID = "0000-0001-8049-7069")),
      person(given = "Benedikt",
             family = "Graeler",
             role = c("aut", "cph"),
             email = "b.graeler@52north.org",
             comment = c(ORCID = "0000-0001-5443-4304")))
License: Apache License
Type: Package
Encoding: UTF-8
VignetteBuilder: knitr
RoxygenNote: 7.3.3
URL: https://r-spatial.github.io/sftime/,
        https://github.com/r-spatial/sftime
BugReports: https://github.com/r-spatial/sftime/issues/
Collate: sftime.R init.R join.R plot.R st_cast.R st_geometry.R
        st_time.R tidyverse.R bind.R crop.R geom-transformers.R
NeedsCompilation: no
Packaged: 2026-05-07 20:02:01 UTC; henni
Author: Henning Teickner [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0002-3993-1182>),
  Edzer Pebesma [aut, cph] (ORCID:
    <https://orcid.org/0000-0001-8049-7069>),
  Benedikt Graeler [aut, cph] (ORCID:
    <https://orcid.org/0000-0001-5443-4304>)
Maintainer: Henning Teickner <henning.teickner@uni-muenster.de>
Repository: CRAN
Date/Publication: 2026-05-09 14:40:48 UTC
Built: R 4.6.0; ; 2026-05-09 18:53:18 UTC; unix
