# Generated by roxygen2: do not edit by hand

S3method("$<-",sftime)
S3method("[",sftime)
S3method("[[<-",sftime)
S3method("st_time<-",sf)
S3method("st_time<-",sftime)
S3method(cbind,sftime)
S3method(plot,sftime)
S3method(print,sftime)
S3method(rbind,sftime)
S3method(st_as_sftime,ST)
S3method(st_as_sftime,Track)
S3method(st_as_sftime,Tracks)
S3method(st_as_sftime,TracksCollection)
S3method(st_as_sftime,cubble_df)
S3method(st_as_sftime,data.frame)
S3method(st_as_sftime,lpp)
S3method(st_as_sftime,ppp)
S3method(st_as_sftime,psp)
S3method(st_as_sftime,sf)
S3method(st_as_sftime,sftime)
S3method(st_as_sftime,sftrack)
S3method(st_as_sftime,sftraj)
S3method(st_as_sftime,stars)
S3method(st_cast,sftime)
S3method(st_crop,sftime)
S3method(st_difference,sftime)
S3method(st_drop_geometry,sftime)
S3method(st_filter,sftime)
S3method(st_intersection,sftime)
S3method(st_join,sftime)
S3method(st_sym_difference,sftime)
S3method(st_time,sftime)
S3method(st_union,sftime)
S3method(transform,sftime)
export("st_time<-")
export(is_sortable)
export(st_as_sftime)
export(st_drop_time)
export(st_set_time)
export(st_sftime)
export(st_time)
import(sf)
importFrom(graphics,plot)
importFrom(methods,as)
importFrom(methods,slotNames)
importFrom(utils,methods)
